#!/usr/bin/python

# Copyright (C) Vladimir Prus 2006.
# Distributed under the Boost Software License, Version 1.0. (See
# accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

# Test the <implicit-dependency> is respected even if the target referred to is
# not built itself, but only referred to by <implicit-dependency>.

import BoostBuild

t = BoostBuild.Tester(use_test_config=False)

t.write("jamroot.jam", """
make a.h : : gen-header ;
explicit a.h ;

exe hello : hello.cpp : <implicit-dependency>a.h ;

import os ;
if [ os.name ] = NT
{
    actions gen-header
    {
       echo int i; > $(<)
    }
}
else
{
    actions gen-header
    {
        echo "int i;" > $(<)
    }
}
""")

t.write("hello.cpp", """
#include "a.h"
int main() { return i; }
""")


t.run_build_system()

t.expect_addition("bin/$toolset/debug*/hello.exe")

t.rm("bin")

t.write("jamroot.jam", """
make dir/a.h : : gen-header ;
explicit dir/a.h ;

exe hello : hello.cpp : <implicit-dependency>dir/a.h ;

import os ;
if [ os.name ] = NT
{
    actions gen-header
    {
       echo int i; > $(<)
    }
}
else
{
    actions gen-header
    {
        echo "int i;" > $(<)
    }
}
""")

t.write("hello.cpp", """
#include "dir/a.h"
int main() { return i; }
""")
t.run_build_system()

t.expect_addition("bin/$toolset/debug*/hello.exe")

t.cleanup()



#!/usr/bin/python

# Copyright 2004 Vladimir Prus
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

# Test that the user can define his own rule that will call built-in main target
# rule and that this will work.

import BoostBuild


t = BoostBuild.Tester(use_test_config=False)

t.write("jamfile.jam", """
my-test : test.cpp ;
""")

t.write("test.cpp", """
int main() {}
""")

t.write("jamroot.jam", """
using testing ;

rule my-test ( name ? : sources + )
{
    name ?= test ;
    unit-test $(name) : $(sources) ;  # /site-config//cppunit /util//testMain ;
}

IMPORT $(__name__) : my-test : : my-test ;
""")

t.run_build_system()
t.expect_addition("bin/$toolset/debug*/test.passed")

t.cleanup()



#!/usr/bin/python

# Copyright 2003 Dave Abrahams
# Copyright 2003, 2004, 2005, 2006 Vladimir Prus
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

# Test usage of searched-libs: one which are found via -l
# switch to the linker/compiler.

import BoostBuild
import os
import string

t = BoostBuild.Tester(use_test_config=False)


# To start with, we have to prepare a library to link with.
t.write("lib/jamroot.jam", "")
t.write("lib/jamfile.jam", "lib test_lib : test_lib.cpp ;")
t.write("lib/test_lib.cpp", """\
#ifdef _WIN32
__declspec(dllexport)
#endif
void foo() {}
""");

t.run_build_system(subdir="lib")
t.expect_addition("lib/bin/$toolset/debug*/test_lib.dll")


# Auto adjusting of suffixes does not work, since we need to
# change dll to lib.
if ( ( os.name == "nt" ) or os.uname()[0].lower().startswith("cygwin") ) and \
    ( BoostBuild.get_toolset() != "gcc" ):
    t.copy("lib/bin/$toolset/debug*/test_lib.implib", "lib/test_lib.implib")
    t.copy("lib/bin/$toolset/debug*/test_lib.dll", "lib/test_lib.dll")
else:
    t.copy("lib/bin/$toolset/debug*/test_lib.dll", "lib/test_lib.dll")


# Test that the simplest usage of searched library works.
t.write("jamroot.jam", "")

t.write("jamfile.jam", """\
import path ;
import project ;
exe main : main.cpp helper ;
lib helper : helper.cpp test_lib ;
lib test_lib : : <name>test_lib <search>lib ;
""")

t.write("main.cpp", """\
void helper();
int main() { helper(); }
""")

t.write("helper.cpp", """\
void foo();
void
#if defined(_WIN32)
__declspec(dllexport)
#endif
helper() { foo(); }
""")

t.run_build_system(["-d2"])
t.expect_addition("bin/$toolset/debug*/main.exe")
t.rm("bin/$toolset/debug/main.exe")
t.rm("bin/$toolset/debug/*/main.exe")


# Test that 'unit-test' will correctly add runtime paths to searched libraries.
t.write("jamfile.jam", """\
import path ;
import project ;
import testing ;

project : requirements <hardcode-dll-paths>false ;

unit-test main : main.cpp helper ;
lib helper : helper.cpp test_lib ;
lib test_lib : : <name>test_lib <search>lib ;
""")

t.run_build_system()
t.expect_addition("bin/$toolset/debug*/main.passed")
t.rm("bin/$toolset/debug/main.exe")
t.rm("bin/$toolset/debug/*/main.exe")


# Now try using searched lib from static lib. Request shared version of searched
# lib, since we do not have a static one handy.
t.write("jamfile.jam", """\
exe main : main.cpp helper ;
lib helper : helper.cpp test_lib/<link>shared : <link>static ;
lib test_lib : : <name>test_lib <search>lib ;
""")

t.run_build_system(stderr=None)
t.expect_addition("bin/$toolset/debug*/main.exe")
t.expect_addition("bin/$toolset/debug/link-static*/helper.lib")
t.rm("bin/$toolset/debug/main.exe")
t.rm("bin/$toolset/debug/*/main.exe")

# A regression test: <library>property referring to searched-lib was being
# mishandled. As the result, we were putting target name to the command line!
# Note that
#    g++ ...... <.>z
# works nicely in some cases, sending output from compiler to file 'z'. This
# problem shows up when searched libs are in usage requirements.
t.write("jamfile.jam", "exe main : main.cpp d/d2//a ;")
t.write("main.cpp", """\
void foo();
int main() { foo(); }
""")

t.write("d/d2/jamfile.jam", """\
lib test_lib : : <name>test_lib <search>../../lib ;
lib a : a.cpp : : : <library>test_lib ;
""")

t.write("d/d2/a.cpp", """\
#ifdef _WIN32
__declspec(dllexport) int force_library_creation_for_a;
#endif
""")

t.run_build_system()


# A regression test. Searched targets were not associated with any properties.
# For that reason, if the same searched lib is generated with two different
# properties, we had an error saying they are actualized to the same Jam target
# name.
t.write("jamroot.jam", "")

t.write("a.cpp", "")

# The 'l' library will be built in two variants: 'debug' (directly requested)
# and 'release' (requested from 'a').
t.write("jamfile.jam", """\
exe a : a.cpp l/<variant>release ;
lib l : : <name>l_d <variant>debug ;
lib l : : <name>l_r <variant>release ;
""")

t.run_build_system(["-n"])


# A regression test. Two virtual target with the same properties were created
# for 'l' target, which caused and error to be reported when actualizing
# targets. The final error is correct, but we should not create two duplicated
# targets. Thanks to Andre Hentz for finding this bug.
t.write("jamroot.jam", "")
t.write("a.cpp", "")
t.write("jamfile.jam", """\
project a : requirements <runtime-link>static ;
static-lib a : a.cpp l ;
lib l : : <name>l_f ;
""")

t.run_build_system(["-n"])


# Make sure plain "lib foobar ; " works.
t.write("jamfile.jam", """\
exe a : a.cpp foobar ;
lib foobar ;
""")

t.run_build_system(["-n", "-d2"])
t.fail_test(string.find(t.stdout(), "foobar") == -1)


# Make sure plain "lib foo bar ; " works.
t.write("jamfile.jam", """\
exe a : a.cpp foo bar ;
lib foo bar ;
""")

t.run_build_system(["-n", "-d2"])
t.fail_test(string.find(t.stdout(), "foo") == -1)
t.fail_test(string.find(t.stdout(), "bar") == -1)

t.cleanup()



#!/usr/bin/python

# Copyright 2016 Steven Watanabe
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

# Test the mi interface for the debugger

import BoostBuild
import TestCmd
import re

def split_stdin_stdout(text):
    """stdin is all text after the prompt up to and including
    the next newline.  Everything else is stdout.  stdout
    may contain regular expressions enclosed in {{}}."""
    prompt = re.escape('(gdb) \n')
    pattern = re.compile('(?<=%s)((?:\d*-.*)\n)' % prompt)
    stdin = ''.join(re.findall(pattern, text))
    stdout = re.sub(pattern, '', text)
    outside_pattern = re.compile(r'(?:\A|(?<=\}\}))(?:[^\{]|(?:\{(?!\{)))*(?:(?=\{\{)|\Z)')

    def escape_line(line):
        line = re.sub(outside_pattern, lambda m: re.escape(m.group(0)), line)
        return re.sub(r'\{\{|\}\}', '', line)

    stdout = '\n'.join([escape_line(line) for line in stdout.split('\n')])
    return (stdin,stdout)

def run(tester, io):
    (input,output) = split_stdin_stdout(io)
    tester.run_build_system(stdin=input, stdout=output, match=TestCmd.match_re)

def make_tester():
    return BoostBuild.Tester(["-dmi"], pass_toolset=False, pass_d0=False,
        use_test_config=False, ignore_toolset_requirements=False, match=TestCmd.match_re)

def test_exec_run():
    t = make_tester()
    t.write("test.jam", """\
        UPDATE ;
    """)

    run(t, """\
=thread-group-added,id="i1"
(gdb) 
72-exec-run -ftest.jam
=thread-created,id="1",group-id="i1"
72^running
(gdb) 
*stopped,reason="exited-normally"
(gdb) 
73-gdb-exit
73^exit
""")

    t.cleanup()

def test_exit_status():
    t = make_tester()
    t.write("test.jam", """\
        EXIT : 1 ;
    """)
    run(t, """\
=thread-group-added,id="i1"
(gdb) 
72-exec-run -ftest.jam
=thread-created,id="1",group-id="i1"
72^running
(gdb) 

*stopped,reason="exited",exit-code="1"
(gdb) 
73-gdb-exit
73^exit
""")
    t.cleanup()

def test_exec_step():
    t = make_tester()
    t.write("test.jam", """\
        rule g ( )
        {
            a = 1 ;
            b = 2 ;
        }
        rule f ( )
        {
            g ;
            c = 3 ;
        }
        f ;
    """)
    run(t, """\
=thread-group-added,id="i1"
(gdb) 
-break-insert f
^done,bkpt={number="1",type="breakpoint",disp="keep",enabled="y",func="f"}
(gdb) 
72-exec-run -ftest.jam
=thread-created,id="1",group-id="i1"
72^running
(gdb) 
*stopped,reason="breakpoint-hit",bkptno="1",disp="keep",frame={func="f",args=[],file="test.jam",fullname="{{.*}}test.jam",line="8"},thread-id="1",stopped-threads="all"
(gdb) 
1-exec-step
1^running
(gdb) 
*stopped,reason="end-stepping-range",frame={func="g",args=[],file="test.jam",fullname="{{.*}}test.jam",line="3"},thread-id="1"
(gdb) 
2-exec-step
2^running
(gdb) 
*stopped,reason="end-stepping-range",frame={func="g",args=[],file="test.jam",fullname="{{.*}}test.jam",line="4"},thread-id="1"
(gdb) 
3-exec-step
3^running
(gdb) 
*stopped,reason="end-stepping-range",frame={func="f",args=[],file="test.jam",fullname="{{.*}}test.jam",line="9"},thread-id="1"
(gdb) 
73-gdb-exit
73^exit
""")
    t.cleanup()
    
def test_exec_next():
    t = make_tester()
    t.write("test.jam", """\
        rule g ( )
        {
            a = 1 ;
        }
        rule f ( )
        {
            g ;
            b = 2 ;
            c = 3 ;
        }
        rule h ( )
        {
            f ;
            g ;
        }
        h ;
        d = 4 ;
    """)
    run(t, """\
=thread-group-added,id="i1"
(gdb) 
-break-insert f
^done,bkpt={number="1",type="breakpoint",disp="keep",enabled="y",func="f"}
(gdb) 
72-exec-run -ftest.jam
=thread-created,id="1",group-id="i1"
72^running
(gdb) 
*stopped,reason="breakpoint-hit",bkptno="1",disp="keep",frame={func="f",args=[],file="test.jam",fullname="{{.*}}test.jam",line="7"},thread-id="1",stopped-threads="all"
(gdb) 
1-exec-next
1^running
(gdb) 
*stopped,reason="end-stepping-range",frame={func="f",args=[],file="test.jam",fullname="{{.*}}test.jam",line="8"},thread-id="1"
(gdb) 
2-exec-next
2^running
(gdb) 
*stopped,reason="end-stepping-range",frame={func="f",args=[],file="test.jam",fullname="{{.*}}test.jam",line="9"},thread-id="1"
(gdb) 
3-exec-next
3^running
(gdb) 
*stopped,reason="end-stepping-range",frame={func="h",args=[],file="test.jam",fullname="{{.*}}test.jam",line="14"},thread-id="1"
(gdb) 
4-exec-next
4^running
(gdb) 
*stopped,reason="end-stepping-range",frame={func="module scope",args=[],file="test.jam",fullname="{{.*}}test.jam",line="17"},thread-id="1"
(gdb) 
73-gdb-exit
73^exit
""")
    t.cleanup()

def test_exec_finish():
    t = make_tester()
    t.write("test.jam", """\
        rule f ( )
        {
            a = 1 ;
        }
        rule g ( )
        {
            f ;
            b = 2 ;
            i ;
        }
        rule h ( )
        {
            g ;
            i ;
        }
        rule i ( )
        {
            c = 3 ;
        }
        h ;
        d = 4 ;
    """)
    run(t, """\
=thread-group-added,id="i1"
(gdb) 
-break-insert f
^done,bkpt={number="1",type="breakpoint",disp="keep",enabled="y",func="f"}
(gdb) 
72-exec-run -ftest.jam
=thread-created,id="1",group-id="i1"
72^running
(gdb) 
*stopped,reason="breakpoint-hit",bkptno="1",disp="keep",frame={func="f",args=[],file="test.jam",fullname="{{.*}}test.jam",line="3"},thread-id="1",stopped-threads="all"
(gdb) 
1-exec-finish
1^running
(gdb) 
*stopped,reason="end-stepping-range",frame={func="g",args=[],file="test.jam",fullname="{{.*}}test.jam",line="8"},thread-id="1"
(gdb) 
2-exec-finish
2^running
(gdb) 
*stopped,reason="end-stepping-range",frame={func="h",args=[],file="test.jam",fullname="{{.*}}test.jam",line="14"},thread-id="1"
(gdb) 
3-exec-finish
3^running
(gdb) 
*stopped,reason="end-stepping-range",frame={func="module scope",args=[],file="test.jam",fullname="{{.*}}test.jam",line="21"},thread-id="1"
(gdb) 
73-gdb-exit
73^exit
""")
    t.cleanup()


def test_breakpoints():
    """Tests the interaction between the following commands:
    break, clear, delete, disable, enable"""
    t = make_tester()
    t.write("test.jam", """\
        rule f ( )
        {
            a = 1 ;
        }
        rule g ( )
        {
            b = 2 ;
        }
        rule h ( )
        {
            c = 3 ;
            d = 4 ;
        }
        f ;
        g ;
        h ;
        UPDATE ;
    """)
    run(t, """\
=thread-group-added,id="i1"
(gdb) 
-break-insert f
^done,bkpt={number="1",type="breakpoint",disp="keep",enabled="y",func="f"}
(gdb) 
72-exec-run -ftest.jam
=thread-created,id="1",group-id="i1"
72^running
(gdb) 
*stopped,reason="breakpoint-hit",bkptno="1",disp="keep",frame={func="f",args=[],file="test.jam",fullname="{{.*}}test.jam",line="3"},thread-id="1",stopped-threads="all"
(gdb) 
-interpreter-exec console kill
^done
(gdb) 
-break-insert g
^done,bkpt={number="2",type="breakpoint",disp="keep",enabled="y",func="g"}
(gdb) 
-break-disable 1
^done
(gdb) 
73-exec-run -ftest.jam
=thread-created,id="1",group-id="i1"
73^running
(gdb) 
*stopped,reason="breakpoint-hit",bkptno="2",disp="keep",frame={func="g",args=[],file="test.jam",fullname="{{.*}}test.jam",line="7"},thread-id="1",stopped-threads="all"
(gdb) 
-interpreter-exec console kill
^done
(gdb) 
-break-enable 1
^done
(gdb) 
74-exec-run -ftest.jam
=thread-created,id="1",group-id="i1"
74^running
(gdb) 
*stopped,reason="breakpoint-hit",bkptno="1",disp="keep",frame={func="f",args=[],file="test.jam",fullname="{{.*}}test.jam",line="3"},thread-id="1",stopped-threads="all"
(gdb) 
-interpreter-exec console kill
^done
(gdb) 
-break-delete 1
^done
(gdb) 
75-exec-run -ftest.jam
=thread-created,id="1",group-id="i1"
75^running
(gdb) 
*stopped,reason="breakpoint-hit",bkptno="2",disp="keep",frame={func="g",args=[],file="test.jam",fullname="{{.*}}test.jam",line="7"},thread-id="1",stopped-threads="all"
(gdb) 
76-gdb-exit
76^exit
""")
    t.cleanup()

test_exec_run()
test_exit_status()
test_exec_step()
test_exec_next()
test_exec_finish()
test_breakpoints()



#!/usr/bin/python

# Copyright 2004 Vladimir Prus.
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

# Test that we load parent projects before loading children.

import BoostBuild

t = BoostBuild.Tester(use_test_config=False)

t.write("jamroot.jam", """\
use-project /child : child ;
ECHO "Setting parent requirements" ;
project : requirements <define>PASS_THE_TEST ;
alias x : child//main ;
""")

t.write("child/jamfile.jam", """\
ECHO "Setting child requirements" ;
project /child ;
exe main : main.cpp ;
""")

t.write("child/main.cpp", """\
#if defined(PASS_THE_TEST)
int main() {}
#endif
""")

t.run_build_system()

t.expect_addition("child/bin/$toolset/debug*/main.exe")
t.fail_test(t.stdout().find("Setting child requirements") < t.stdout().find(
    "Setting parent requirements"))


# Regression test: parent requirements were ignored in some cases.
t.rm(".")
t.write("jamroot.jam", "build-project src ;")
t.write("src/jamfile.jam", "project : requirements <define>EVERYTHING_OK ;")
t.write("src/app/jamfile.jam", "exe test : test.cpp ;")
t.write("src/app/test.cpp", """\
#ifdef EVERYTHING_OK
int main() {}
#endif
""")

t.run_build_system(subdir="src/app")
t.expect_addition("src/app/bin/$toolset/debug*/test.exe")


# child/child2 used to be loaded before child
t.rm(".")
t.write("jamroot.jam", """\
use-project /child/child2 : child/child2 ;
rule parent-rule ( )
{
  ECHO "Running parent-rule" ;
}
""")
t.write("child/jamfile.jam", "")
t.write("child/child1/jamfile.jam", "")
t.write("child/child2/jamfile.jam", "parent-rule ;")

t.run_build_system(subdir="child/child1")
t.expect_output_lines("Running parent-rule")

t.cleanup()



#!/usr/bin/python

# Copyright (C) Vladimir Prus 2006.
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

# Tests for the target id resolution process.

import BoostBuild

# Create a temporary working directory.
t = BoostBuild.Tester(use_test_config=False)

# Create the needed files
t.write("jamroot.jam", """\
exe hello : hello.cpp ;
# This should use the 'hello' target, even if there is a 'hello' file in the
# current dir.
install s : hello : <location>. ;
""")

t.write("hello.cpp", "int main() {}\n")

t.run_build_system()

t.expect_addition("bin/$toolset/debug*/hello.obj")

t.touch("hello.cpp")
t.run_build_system(["s"])
# If 'hello' in the 's' target resolved to file in the current dir, nothing
# will be rebuilt.
t.expect_touch("bin/$toolset/debug*/hello.obj")

t.cleanup()



# Copyright 2003 Dave Abrahams
# Copyright 2001, 2002 Vladimir Prus
# Copyright 2012 Jurko Gospodnetic
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

###############################################################################
#
# Based in part on an old Subversion tree.py source file (tools for comparing
# directory trees). See http://subversion.tigris.org for more information.
#
# Copyright (c) 2001 Sam Tobin-Hochstadt.  All rights reserved.
#
# This software is licensed as described in the file COPYING, which you should
# have received as part of this distribution. The terms are also available at
# http://subversion.tigris.org/license-1.html. If newer versions of this
# license are posted there, you may use a newer version instead, at your
# option.
#
###############################################################################

import os
import os.path
import stat
import sys


class TreeNode:
    """
      Fundamental data type used to build file system tree structures.

      If CHILDREN is None, then the node represents a file. Otherwise, CHILDREN
    is a list of the nodes representing that directory's children.

      NAME is simply the name of the file or directory. CONTENTS is a string
    holding the file's contents (if a file).

    """

    def __init__(self, name, children=None, contents=None):
        assert children is None or contents is None
        self.name = name
        self.mtime = 0
        self.children = children
        self.contents = contents
        self.path = name

    def add_child(self, newchild):
        assert not self.is_file()
        for a in self.children:
            if a.name == newchild.name:
                if newchild.is_file():
                    a.contents = newchild.contents
                    a.path = os.path.join(self.path, newchild.name)
                else:
                    for i in newchild.children:
                        a.add_child(i)
                break
        else:
            self.children.append(newchild)
            newchild.path = os.path.join(self.path, newchild.name)

    def get_child(self, name):
        """
          If the given TreeNode directory NODE contains a child named NAME,
        return the child; else, return None.

        """
        for n in self.children:
            if n.name == name:
                return n

    def is_file(self):
        return self.children is None

    def pprint(self):
        print(" * Node name: %s" % self.name)
        print("    Path:     %s" % self.path)
        print("    Contents: %s" % self.contents)
        if self.is_file():
            print("    Children: is a file.")
        else:
            print("    Children: %d" % len(self.children))


class TreeDifference:
    def __init__(self):
        self.added_files = []
        self.removed_files = []
        self.modified_files = []
        self.touched_files = []

    def append(self, other):
        self.added_files.extend(other.added_files)
        self.removed_files.extend(other.removed_files)
        self.modified_files.extend(other.modified_files)
        self.touched_files.extend(other.touched_files)

    def ignore_directories(self):
        """Removes directories from our lists of found differences."""
        not_dir = lambda x : x[-1] != "/"
        self.added_files = filter(not_dir, self.added_files)
        self.removed_files = filter(not_dir, self.removed_files)
        self.modified_files = filter(not_dir, self.modified_files)
        self.touched_files = filter(not_dir, self.touched_files)

    def pprint(self, file=sys.stdout):
        file.write("Added files   : %s\n" % self.added_files)
        file.write("Removed files : %s\n" % self.removed_files)
        file.write("Modified files: %s\n" % self.modified_files)
        file.write("Touched files : %s\n" % self.touched_files)

    def empty(self):
        return not (self.added_files or self.removed_files or
            self.modified_files or self.touched_files)


def build_tree(path):
    """
      Takes PATH as the folder path, walks the file system below that path, and
    creates a tree structure based on any files and folders found there.
    Returns the prepared tree structure plus the maximum file modification
    timestamp under the given folder.

    """
    return _handle_dir(os.path.normpath(path))


def tree_difference(a, b):
    """Compare TreeNodes A and B, and create a TreeDifference instance."""
    return _do_tree_difference(a, b, "", True)


def _do_tree_difference(a, b, parent_path, root=False):
    """Internal recursive worker function for tree_difference()."""

    # We do not want to list root node names.
    if root:
        assert not parent_path
        assert not a.is_file()
        assert not b.is_file()
        full_path = ""
    else:
        assert a.name == b.name
        full_path = parent_path + a.name
    result = TreeDifference()

    # A and B are both files.
    if a.is_file() and b.is_file():
        if a.contents != b.contents:
            result.modified_files.append(full_path)
        elif a.mtime != b.mtime:
            result.touched_files.append(full_path)
        return result

    # Directory converted to file.
    if not a.is_file() and b.is_file():
        result.removed_files.extend(_traverse_tree(a, parent_path))
        result.added_files.append(full_path)

    # File converted to directory.
    elif a.is_file() and not b.is_file():
        result.removed_files.append(full_path)
        result.added_files.extend(_traverse_tree(b, parent_path))

    # A and B are both directories.
    else:
        if full_path:
            full_path += "/"
        accounted_for = []  # Children present in both trees.
        for a_child in a.children:
            b_child = b.get_child(a_child.name)
            if b_child:
                accounted_for.append(b_child)
                result.append(_do_tree_difference(a_child, b_child, full_path))
            else:
                result.removed_files.append(full_path + a_child.name)
        for b_child in b.children:
            if b_child not in accounted_for:
                result.added_files.extend(_traverse_tree(b_child, full_path))

    return result


def _traverse_tree(t, parent_path):
    """Returns a list of all names in a tree."""
    assert not parent_path or parent_path[-1] == "/"
    full_node_name = parent_path + t.name
    if t.is_file():
        result = [full_node_name]
    else:
        name_prefix = full_node_name + "/"
        result = [name_prefix]
        for i in t.children:
            result.extend(_traverse_tree(i, name_prefix))
    return result


def _get_text(path):
    """Return a string with the textual contents of a file at PATH."""
    fp = open(path, 'r')
    try:
        return fp.read()
    finally:
        fp.close()


def _handle_dir(path):
    """
      Main recursive worker function for build_tree(). Returns a newly created
    tree node representing the given normalized folder path as well as the
    maximum file/folder modification time detected under the same path.

    """
    files = []
    dirs = []
    node = TreeNode(os.path.basename(path), children=[])
    max_mtime = node.mtime = os.stat(path).st_mtime

    # List files & folders.
    for f in os.listdir(path):
        f = os.path.join(path, f)
        if os.path.isdir(f):
            dirs.append(f)
        elif os.path.isfile(f):
            files.append(f)

    # Add a child node for each file.
    for f in files:
        fcontents = _get_text(f)
        new_file_node = TreeNode(os.path.basename(f), contents=fcontents)
        new_file_node.mtime = os.stat(f).st_mtime
        max_mtime = max(max_mtime, new_file_node.mtime)
        node.add_child(new_file_node)

    # For each subdir, create a node, walk its tree, add it as a child.
    for d in dirs:
        new_dir_node, new_max_mtime = _handle_dir(d)
        max_mtime = max(max_mtime, new_max_mtime)
        node.add_child(new_dir_node)

    return node, max_mtime



#!/usr/bin/python

# Copyright 2001 Dave Abrahams
# Copyright 2011 Steven Watanabe
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

# Tests Windows command line construction.
#
# Note that the regular 'echo' is an internal shell command on Windows and
# therefore can not be called directly as a standalone Windows process.

import BoostBuild
import os
import re
import sys


executable = sys.executable.replace("\\", "/")
if " " in executable:
    executable = '"%s"' % executable


def string_of_length(n):
    if n <= 0:
        return ""
    n -= 1
    y = ['', '$(1x10-1)', '$(10x10-1)', '$(100x10-1)', '$(1000x10-1)']
    result = []
    for i in reversed(xrange(5)):
        x, n = divmod(n, 10 ** i)
        result += [y[i]] * x
    result.append('x')
    return " ".join(result)


#   Boost Jam currently does not allow preparing actions with completely empty
# content and always requires at least a single whitespace after the opening
# brace in order to satisfy its Boost Jam language grammar rules.
def test_raw_empty():
    whitespace_in = "  \n\n\r\r\v\v\t\t   \t   \r\r   \n\n"

    #   We tell the testing system to read its child process output as raw
    # binary data but the bjam process we run will read its input file and
    # write out its output as text, i.e. convert all of our "\r\n" sequences to
    # "\n" on input and all of its "\n" characters back to "\r\n" on output.
    # This means that any lone "\n" input characters not preceded by "\r" will
    # get an extra "\r" added in front of it on output.
    whitespace_out = whitespace_in.replace("\r\n", "\n").replace("\n", "\r\n")

    t = BoostBuild.Tester(["-d2", "-d+4"], pass_d0=False, pass_toolset=0,
        use_test_config=False)
    t.write("file.jam", """\
actions do_empty {%s}
JAMSHELL = %% ;
do_empty all ;
""" % (whitespace_in))
    t.run_build_system(["-ffile.jam"], universal_newlines=False)
    t.expect_output_lines("do_empty all")
    t.expect_output_lines("Executing raw command directly", False)
    if "\r\n%s\r\n" % whitespace_out not in t.stdout():
        BoostBuild.annotation("failure", "Whitespace action content not found "
            "on stdout.")
        t.fail_test(1, dump_difference=False)
    t.cleanup()


def test_raw_nt(n=None, error=False):
    t = BoostBuild.Tester(["-d1", "-d+4"], pass_d0=False, pass_toolset=0,
        use_test_config=False)

    cmd_prefix = "%s -c \"print('XXX: " % executable
    cmd_suffix = "')\""
    cmd_extra_length = len(cmd_prefix) + len(cmd_suffix)

    if n == None:
        n = cmd_extra_length

    data_length = n - cmd_extra_length
    if data_length < 0:
        BoostBuild.annotation("failure", """\
Can not construct Windows command of desired length. Requested command length
too short for the current test configuration.
    Requested command length: %d
    Minimal supported command length: %d
""" % (n, cmd_extra_length))
        t.fail_test(1, dump_difference=False)

    #   Each $(Xx10-1) variable contains X words of 9 characters each, which,
    # including spaces between words, brings the total number of characters in
    # its string representation to X * 10 - 1 (X * 9 characters + (X - 1)
    # spaces).
    t.write("file.jam", """\
ten = 0 1 2 3 4 5 6 7 8 9 ;

1x10-1 = 123456789 ;
10x10-1 = $(ten)12345678 ;
100x10-1 = $(ten)$(ten)1234567 ;
1000x10-1 = $(ten)$(ten)$(ten)123456 ;

actions do_echo
{
    %s%s%s
}
JAMSHELL = %% ;
do_echo all ;
""" % (cmd_prefix, string_of_length(data_length), cmd_suffix))
    if error:
        expected_status = 1
    else:
        expected_status = 0
    t.run_build_system(["-ffile.jam"], status=expected_status)
    if error:
        t.expect_output_lines("Executing raw command directly", False)
        t.expect_output_lines("do_echo action is too long (%d, max 32766):" % n
            )
        t.expect_output_lines("XXX: *", False)
    else:
        t.expect_output_lines("Executing raw command directly")
        t.expect_output_lines("do_echo action is too long*", False)

        m = re.search("^XXX: (.*)$", t.stdout(), re.MULTILINE)
        if not m:
            BoostBuild.annotation("failure", "Expected output line starting "
                "with 'XXX: ' not found.")
            t.fail_test(1, dump_difference=False)
        if len(m.group(1)) != data_length:
            BoostBuild.annotation("failure", """Unexpected output data length.
    Expected: %d
    Received: %d""" % (n, len(m.group(1))))
            t.fail_test(1, dump_difference=False)

    t.cleanup()


def test_raw_to_shell_fallback_nt():
    t = BoostBuild.Tester(["-d1", "-d+4"], pass_d0=False, pass_toolset=0,
        use_test_config=False)

    cmd_prefix = '%s -c print(' % executable
    cmd_suffix = ')'

    t.write("file_multiline.jam", """\
actions do_multiline
{
    echo one


    echo two
}
JAMSHELL = % ;
do_multiline all ;
""")
    t.run_build_system(["-ffile_multiline.jam"])
    t.expect_output_lines("do_multiline all")
    t.expect_output_lines("one")
    t.expect_output_lines("two")
    t.expect_output_lines("Executing raw command directly", False)
    t.expect_output_lines("Executing using a command file and the shell: "
        "cmd.exe /Q/C")

    t.write("file_redirect.jam", """\
actions do_redirect { echo one > two.txt }
JAMSHELL = % ;
do_redirect all ;
""")
    t.run_build_system(["-ffile_redirect.jam"])
    t.expect_output_lines("do_redirect all")
    t.expect_output_lines("one", False)
    t.expect_output_lines("Executing raw command directly", False)
    t.expect_output_lines("Executing using a command file and the shell: "
        "cmd.exe /Q/C")
    t.expect_addition("two.txt")

    t.write("file_pipe.jam", """\
actions do_pipe
{
    echo one | echo two
}
JAMSHELL = % ;
do_pipe all ;
""")
    t.run_build_system(["-ffile_pipe.jam"])
    t.expect_output_lines("do_pipe all")
    t.expect_output_lines("one*", False)
    t.expect_output_lines("two")
    t.expect_output_lines("Executing raw command directly", False)
    t.expect_output_lines("Executing using a command file and the shell: "
        "cmd.exe /Q/C")

    t.write("file_single_quoted.jam", """\
actions do_single_quoted { %s'5>10'%s }
JAMSHELL = %% ;
do_single_quoted all ;
""" % (cmd_prefix, cmd_suffix))
    t.run_build_system(["-ffile_single_quoted.jam"])
    t.expect_output_lines("do_single_quoted all")
    t.expect_output_lines("5>10")
    t.expect_output_lines("Executing raw command directly")
    t.expect_output_lines("Executing using a command file and the shell: "
        "cmd.exe /Q/C", False)
    t.expect_nothing_more()

    t.write("file_double_quoted.jam", """\
actions do_double_quoted { %s"5>10"%s }
JAMSHELL = %% ;
do_double_quoted all ;
""" % (cmd_prefix, cmd_suffix))
    t.run_build_system(["-ffile_double_quoted.jam"])
    t.expect_output_lines("do_double_quoted all")
    # The difference between this example and the similar previous one using
    # single instead of double quotes stems from how the used Python executable
    # parses the command-line string received from Windows.
    t.expect_output_lines("False")
    t.expect_output_lines("Executing raw command directly")
    t.expect_output_lines("Executing using a command file and the shell: "
        "cmd.exe /Q/C", False)
    t.expect_nothing_more()

    t.write("file_escaped_quote.jam", """\
actions do_escaped_quote { %s\\"5>10\\"%s }
JAMSHELL = %% ;
do_escaped_quote all ;
""" % (cmd_prefix, cmd_suffix))
    t.run_build_system(["-ffile_escaped_quote.jam"])
    t.expect_output_lines("do_escaped_quote all")
    t.expect_output_lines("5>10")
    t.expect_output_lines("Executing raw command directly", False)
    t.expect_output_lines("Executing using a command file and the shell: "
        "cmd.exe /Q/C")
    t.expect_nothing_more()

    t.cleanup()


###############################################################################
#
# main()
# ------
#
###############################################################################

if os.name == 'nt':
    test_raw_empty()

    #   Can not test much shorter lengths as the shortest possible command line
    # line length constructed in this depends on the runtime environment, e.g.
    # path to the Panther executable running this test.
    test_raw_nt()
    test_raw_nt(255)
    test_raw_nt(1000)
    test_raw_nt(8000)
    test_raw_nt(8191)
    test_raw_nt(8192)
    test_raw_nt(10000)
    test_raw_nt(30000)
    test_raw_nt(32766)
    #   CreateProcessA() Windows API places a limit of 32768 on the allowed
    # command-line length, including a trailing Unicode (2-byte) nul-terminator
    # character.
    test_raw_nt(32767, error=True)
    test_raw_nt(40000, error=True)
    test_raw_nt(100001, error=True)

    test_raw_to_shell_fallback_nt()


#!/usr/bin/python

# Copyright 2003, 2004, 2005 Vladimir Prus
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

import BoostBuild

# Create a temporary working directory.
t = BoostBuild.Tester()

# Create the needed files.
t.write("jamroot.jam", """\
constant FOO : foobar gee ;
ECHO $(FOO) ;
""")

t.run_build_system()
t.expect_output_lines("foobar gee")

# Regression test: when absolute paths were passed to path-constant rule,
# Boost.Build failed to recognize path as absolute and prepended the current
# dir.
t.write("jamroot.jam", """\
import path ;
local here = [ path.native [ path.pwd ] ] ;
path-constant HERE : $(here) ;
if $(HERE) != $(here)
{
    ECHO "PWD           =" $(here) ;
    ECHO "path constant =" $(HERE) ;
    EXIT ;
}
""")
t.write("jamfile.jam", "")

t.run_build_system()

t.write("jamfile.jam", """\
# This tests that rule 'hello' will be imported to children unlocalized, and
# will still access variables in this Jamfile.
x = 10 ;
constant FOO : foo ;
rule hello ( ) { ECHO "Hello $(x)" ; }
""")

t.write("d/jamfile.jam", """\
ECHO "d: $(FOO)" ;
constant BAR : bar ;
""")

t.write("d/d2/jamfile.jam", """\
ECHO "d2: $(FOO)" ;
ECHO "d2: $(BAR)" ;
hello ;
""")

t.run_build_system(subdir="d/d2")
t.expect_output_lines("d: foo\nd2: foo\nd2: bar\nHello 10")

t.cleanup()



import unittest

import numpy as np
from pystan import StanModel
from pystan.tests.helper import get_model

class TestMatrixParam(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        model_code = """
        data {
        int<lower=2> K;
        int<lower=1> D;
        }
        parameters {
        matrix[K,D] beta;
        }
        model {
        for (k in 1:K)
            for (d in 1:D)
            beta[k,d] ~ normal(if_else(d==2,100, 0),1);
        }"""
        cls.model = get_model("matrix_normal_model", model_code)
        #cls.model = StanModel(model_code=model_code)

    def test_matrix_param(self):
        sm = self.model
        fit = sm.sampling(data=dict(K=3, D=4))
        beta = fit.extract()['beta']
        assert beta.shape == (4000, 3, 4)
        assert np.mean(beta[:, 0, 0]) < 4
        extracted = fit.extract(permuted=False)
        assert extracted.shape == (1000, 4, 13)
        assert np.mean(extracted[:,:,0]) < 4
        assert np.all(extracted[:,:,12] < 0)  # lp__

    def test_matrix_param_order(self):
        sm = self.model
        fit = sm.sampling(data=dict(K=3, D=2))
        beta = fit.extract()['beta']
        assert beta.shape == (4000, 3, 2)
        beta_mean = np.mean(beta, axis=0)
        beta_colmeans = np.mean(beta_mean, axis=0)
        assert beta_colmeans[0] < 4
        assert beta_colmeans[1] > 100 - 4

    def test_matrix_param_order_optimizing(self):
        sm = self.model
        op = sm.optimizing(data=dict(K=3, D=2))
        beta = op['beta']
        assert beta.shape == (3, 2)
        beta_colmeans = np.mean(beta, axis=0)
        assert beta_colmeans[0] < 4
        assert beta_colmeans[1] > 100 - 4



#!/usr/bin/python

# Copyright (C) 2013 Steven Watanabe
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

import BoostBuild
import MockToolset

t = BoostBuild.Tester(arguments=['toolset=mock', '--ignore-site-config', '--user-config='], pass_toolset=0)

MockToolset.create(t)

# Build from source
t.write("libpng/png.h", 'libpng')
t.write("libpng/png.c", 'png')

t.write("Jamroot.jam", """
path-constant here : . ;
using libpng : : <source>$(here)/libpng ;
alias libpng : /libpng//libpng : : <link>static <link>shared ;
""")

MockToolset.set_expected(t, '''
source_file('png.c', 'png')
action('-c -x c -I./libpng -o $png.o $png.c')
action('--dll $png.o -o $png.so')
action('--archive $png.o -o $png.a')
''')

t.run_build_system()
t.expect_addition('bin/standalone/libpng/mock/debug/png.dll')
t.expect_addition('bin/standalone/libpng/mock/debug/link-static/png.lib')

t.rm('libpng')

# Generic definitions that aren't configuration specific
common_stuff = '''
source_file('test.cpp', 'test.cpp')
source_file('main.cpp', 'int main() {}')
source_file('png.h.cpp', '#include <png.h>')
action('-c -x c++ $main.cpp -o $main.o')
'''
t.write('test.cpp', 'test.cpp')

# Default initialization - static library
t.rm('bin')
t.write("Jamroot.jam", """
path-constant here : . ;
using libpng ;
exe test : test.cpp /libpng//libpng : : <link>static <link>shared ;
""")

MockToolset.set_expected(t, common_stuff + '''
action('$main.o --static-lib=png -o $config.exe')
action('-c -x c++ $png.h.cpp -o $png.h.o')
action('-c -x c++ $test.cpp -o $test.o')
action('$test.o --static-lib=png -o $test')
''')
t.run_build_system()
t.expect_addition('bin/mock/debug/test.exe')
t.expect_addition('bin/mock/debug/link-static/test.exe')

# Default initialization - shared library
t.rm('bin')
t.write("Jamroot.jam", """
path-constant here : . ;
using libpng ;
exe test : test.cpp /libpng//libpng : : <link>static <link>shared ;
""")

MockToolset.set_expected(t, common_stuff + '''
action('$main.o --shared-lib=png -o $config.exe')
action('-c -x c++ $png.h.cpp -o $png.h.o')
action('-c -x c++ $test.cpp -o $test.o')
action('$test.o --shared-lib=png -o $test')
''')
t.run_build_system()
t.expect_addition('bin/mock/debug/test.exe')
t.expect_addition('bin/mock/debug/link-static/test.exe')

# Initialization in explicit location - static library
t.rm('bin')
t.write("Jamroot.jam", """
path-constant here : . ;
using libpng : : <name>mylibpng <include>$(here)/libpng <search>$(here)/libpng ;
exe test : test.cpp /libpng//libpng : : <link>static <link>shared ;
""")

t.write('libpng/png.h', 'libpng')

MockToolset.set_expected(t, common_stuff + '''
action('$main.o -L./libpng --static-lib=mylibpng -o $config.exe')
action('-c -x c++ $test.cpp -I./libpng -o $test.o')
action('$test.o -L./libpng --static-lib=mylibpng -o $test')
''')
t.run_build_system()
t.expect_addition('bin/mock/debug/test.exe')
t.expect_addition('bin/mock/debug/link-static/test.exe')

# Initialization in explicit location - shared library
t.rm('bin')
t.write("Jamroot.jam", """
path-constant here : . ;
using libpng : : <name>mylibpng <include>$(here)/libpng <search>$(here)/libpng ;
exe test : test.cpp /libpng//libpng : : <link>static <link>shared ;
""")

MockToolset.set_expected(t, common_stuff + '''
action('$main.o -L./libpng --shared-lib=mylibpng -o $config.exe')
action('-c -x c++ $test.cpp -I./libpng -o $test.o')
action('$test.o -L./libpng --shared-lib=mylibpng -o $test')
''')
t.run_build_system()
t.expect_addition('bin/mock/debug/test.exe')
t.expect_addition('bin/mock/debug/link-static/test.exe')

t.cleanup()
